/** 
 * XMLSec library
 *
 * Simple X509 Manager
 *
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_X509MNGR_H__
#define __XMLSEC_X509MNGR_H__    

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 

#include <openssl/x509.h>

#include <xmlsec/xmlsec.h>


/** 
 * These simple functions are provided as an example. However,
 * in most case your can use it as is.
 */
typedef struct _xmlSecSimpleX509Mngr {
    /* type: sign or verify */
    int			sign;
    
    /* verify */
    X509_STORE		*xst;
    
    /* sign */
    EVP_PKEY 		*key;
    STACK_OF(X509)	*signCerts;
    const EVP_MD	*signMD;    
} xmlSecSimpleX509Mngr, *xmlSecSimpleX509MngrPtr;

xmlSecSimpleX509MngrPtr	xmlSecSimpleX509MngrCreate	(int sign);
void		xmlSecSimpleX509MngrDestroy		(xmlSecSimpleX509MngrPtr mngr);
int		xmlSecSimpleX509MngrAddTrustedCA	(xmlSecSimpleX509MngrPtr mngr,
							 const char *filename);
int		xmlSecSimpleX509MngrAddSignCA		(xmlSecSimpleX509MngrPtr mngr,
							 const char *filename);
int		xmlSecSimpleX509MngrVerify		(xmlSecSimpleX509MngrPtr mngr,
							 X509 *cert,
							 STACK_OF(X509) *untrusted);
int		xmlSecSimpleX509MngrAddCRL		(xmlSecSimpleX509MngrPtr mngr,
							 X509_CRL *crl);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XMLSEC_X509MNGR_H__ */

