/** 
 * XMLSec library
 *
 * Simple X509 Context Implementation
 *
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>

#include <xmlsec/xmlsec.h>
#include <xmlsec/keys.h>
#include <xmlsec/x509.h>

#include <xmlsec/x509mngr.h>

    /* verify */
    X509_STORE		*xst;
    
    /* sign */
    EVP_PKEY 		*key;
    STACK_OF(X509)	*signCerts;
    const EVP_MD	*signMD;    

/**
 * xmlSecSimpleX509MngrCreate
 * @sign:
 *
 *
 *
 */
xmlSecSimpleX509MngrPtr	
xmlSecSimpleX509MngrCreate(int sign) {
    static const char func[] = "xmlSecSimpleX509MngrCreate";
    xmlSecSimpleX509MngrPtr mngr;
    
    mngr = (xmlSecSimpleX509MngrPtr) xmlMalloc(sizeof(xmlSecSimpleX509Mngr));
    if(mngr == NULL) {
#ifdef DEBUG_XMLSEC
        xmlGenericError(xmlGenericErrorContext,
	    "%s: xmlSecSimpleX509Mngr malloc failed\n",
	    func);	
#endif 	    
	return(NULL);
    }
    memset(mngr, 0, sizeof(xmlSecSimpleX509Mngr));

    mngr->sign = sign;
    if(sign) {
	/* TODO: */
    } else {
	mngr->xst = X509_STORE_new();
	if(mngr->xst == NULL) {
#ifdef DEBUG_XMLSEC
    	    xmlGenericError(xmlGenericErrorContext,
		"%s: failed to create x509 store\n",
		func);	
#endif 	    
	    xmlSecSimpleX509MngrDestroy(mngr);
	    return(NULL);
	}
        if(!X509_STORE_set_default_paths(mngr->xst)) {
#ifdef DEBUG_XMLSEC
    	    xmlGenericError(xmlGenericErrorContext,
		"%s: failed to set default paths\n",
		func);	
#endif 	    
	    xmlSecSimpleX509MngrDestroy(mngr);
	    return(NULL);
	}	
    }    
    return(mngr);
}

/**
 * xmlSecSimpleX509MngrPtr
 *
 *
 *
 *
 */
void		
xmlSecSimpleX509MngrDestroy(xmlSecSimpleX509MngrPtr mngr) {
    static const char func[] = "xmlSecSimpleX509MngrPtr";
    
    if(mngr == NULL) {
#ifdef DEBUG_XMLSEC
        xmlGenericError(xmlGenericErrorContext,
	    "%s: manager is null \n",
	    func);	
#endif 	    
	return;
    }
    if(mngr->xst != NULL) {
	X509_STORE_free(mngr->xst);
    }
    if(mngr->key != NULL) {
	/* TODO: */
    }
    if(mngr->signCerts != NULL) {
	/* TODO: */
    }
    if(mngr->signMD != NULL) {
	/* TODO: */
    }
    memset(mngr, 0, sizeof(xmlSecSimpleX509Mngr)); 
    xmlFree(mngr);
}

/**
 * xmlSecSimpleX509MngrAddTrustedCA
 *
 *
 *
 *
 */
int
xmlSecSimpleX509MngrAddTrustedCA(xmlSecSimpleX509MngrPtr mngr, const char *filename) {
    static const char func[] = "xmlSecSimpleX509MngrAddTrustedCA";
    X509_LOOKUP *lookup = NULL; 
    int ret;
    
    if((mngr == NULL) || (mngr->xst == NULL) || (filename == NULL)){
#ifdef DEBUG_XMLSEC
        xmlGenericError(xmlGenericErrorContext,
	    "%s: manager, certs store or filename is null \n",
	    func);	
#endif 	    
	return(-1);
    }
    
    lookup = X509_STORE_add_lookup(mngr->xst, X509_LOOKUP_file());
    if(lookup == NULL) {
#ifdef DEBUG_XMLSEC
        xmlGenericError(xmlGenericErrorContext,
	    "%s: x509 file lookup creation failed\n",
	    func);	
#endif 	    
	return(-1);
    }

    ret = X509_LOOKUP_load_file(lookup, filename, X509_FILETYPE_PEM);
    if(ret != 1) {
#ifdef DEBUG_XMLSEC
        xmlGenericError(xmlGenericErrorContext,
	    "%s: x509 file \"%s\" load failed\n", 
	    func, filename);	
#endif 	    
	return(-1);
    }
    return(0);
}

/**
 * xmlSecSimpleX509MngrAddSignCA
 *
 *
 *
 *
 */
int
xmlSecSimpleX509MngrAddSignCA(xmlSecSimpleX509MngrPtr mngr, const char *filename) {
    static const char func[] = "xmlSecSimpleX509MngrAddSignCA";
    /* TODO */
    return(0);
}

/**
 * xmlSecSimpleX509MngrVerify
 *
 *
 *
 *
 */
int
xmlSecSimpleX509MngrVerify(xmlSecSimpleX509MngrPtr mngr, X509 *cert,
			 STACK_OF(X509) *untrusted) {
    static const char func[] = "xmlSecSimpleX509MngrVerify";
    X509_STORE_CTX xsc; 
    int ret;  
    
    if((cert == NULL) || (mngr == NULL)){
#ifdef DEBUG_XMLSEC
        xmlGenericError(xmlGenericErrorContext,
	    "%s: X509_STORE or cert is null\n",
	    func);
#endif	    	
	return(-1);
    }

    X509_STORE_CTX_init (&xsc, mngr->xst, cert, untrusted);
    ret = X509_verify_cert(&xsc); 
    X509_STORE_CTX_cleanup (&xsc);  
    return((ret == 1) ? 1 : 0);
}

/**
 * xmlSecSimpleX509MngrAddCRL
 *
 *
 *
 *
 */
int
xmlSecSimpleX509MngrAddCRL(xmlSecSimpleX509MngrPtr mngr, X509_CRL *crl) {
    static const char func[] = "xmlSecSimpleX509MngrAddCRL";
    
    /* TODO */
#if 0
    if((crl == NULL) || (ctx == NULL)){
        xmlGenericError(xmlGenericErrorContext,
	    "%s: X509_STORE or cert is null\n",
	    func);	
	return(-1);
    }
    X509_STORE_add_crl(ctx, crl);
#endif
    return(0);
}



