#!/bin/sh

topfolder=`pwd`
timestamp=`date +%Y%m%d_%H%M%S` 
tmpfile=/tmp/testDSig.$timestamp-$$.tmp
logfile=/tmp/testDSig.$timestamp-$$.log
script="$0"
keysfile=$topfolder/keys.xml

printRes() {
    if [ $? = 0 ]; then
	echo "   OK"
    else 
        echo " Fail"
    fi
    if [ -f .memdump ] ; then 
	cat .memdump >> $logfile 
    fi
}

execDSigTest() {    
    file=$topfolder/$1      
    echo $1
    
    printf "    Verify existing signature                            "
    $topfolder/testDSig verify $2 $file.xml >> $logfile 2>> $logfile
    printRes 

    printf "    Create new signature                                 "
    $topfolder/testDSig sign $3 $file.tmpl > $tmpfile 2>> $logfile
    printRes

    printf "    Verify new signature                                 "
    $topfolder/testDSig verify $4 $tmpfile >> $logfile 2>> $logfile
    printRes

}

echo "--- testDSig started ($timestamp)"
echo "--- testDSig started ($timestamp)" >> $logfile

echo "-------------------------- ATTENTION --------------------------------"
echo "--- Some of these tests require HTTP connection and will fail     ---" 
echo "--- if the connection could not be established (for example, if   ---"
echo "--- you are not connected to Internet or the target site is down) ---"
echo "-------------------------- ATTENTION --------------------------------"


execDSigTest "merlin-xmldsig-fifteen/signature-enveloped-dsa" \
    "" \
    "--privkey keys/dsakey.pem" \
    "" 
    
execDSigTest "merlin-xmldsig-fifteen/signature-enveloping-b64-dsa" \
    "" \
    "--privkey keys/dsakey.pem" \
    "" 
    
execDSigTest "merlin-xmldsig-fifteen/signature-enveloping-dsa" \
    "" \
    "--privkey keys/dsakey.pem" \
    "" 
    
execDSigTest "merlin-xmldsig-fifteen/signature-enveloping-hmac-sha1-40" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" 
    
execDSigTest "merlin-xmldsig-fifteen/signature-enveloping-hmac-sha1" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" 

execDSigTest "merlin-xmldsig-fifteen/signature-enveloping-rsa" \
    "" \
    "--privkey keys/rsakey.pem" \
    "" 
    
execDSigTest "merlin-xmldsig-fifteen/signature-external-b64-dsa" \
    "" \
    "--privkey keys/dsakey.pem" \
    "" 
    
execDSigTest "merlin-xmldsig-fifteen/signature-external-dsa" \
    "" \
    "--privkey keys/dsakey.pem" \
    "" 

execDSigTest "merlin-xmldsig-fifteen/signature-enveloped-dsa" \
    "" \
    "--privkey keys/dsakey.pem" \
    ""

execDSigTest "merlin-xmldsig-sixteen/signature" \
    "--CAfile $topfolder/keys/merlincert.pem --keys-origins retrieval-doc,x509" \
    "--privkey $topfolder/keys/dsakey.pem,$topfolder/keys/dsacert.pem,$topfolder/keys/ca2cert.pem" \
    "--CAfile $topfolder/keys/cacert.pem --keys-origins retrieval-doc,x509"
    
execDSigTest "merlin-exc-c14n-one/exc-signature" \
    "" \
    "--privkey keys/dsakey.pem" \
    "" 

execDSigTest "aleksey-xmldsig-01/enveloping-dsa-x509chain" \
    "--CAfile $topfolder/keys/cacert.pem --keys-origins x509" \
    "--privkey $topfolder/keys/dsakey.pem,$topfolder/keys/dsacert.pem,$topfolder/keys/ca2cert.pem" \
    "--CAfile $topfolder/keys/cacert.pem --keys-origins x509"

execDSigTest "aleksey-xmldsig-01/enveloping-rsa-x509chain" \
    "--CAfile $topfolder/keys/cacert.pem --keys-origins x509" \
    "--privkey $topfolder/keys/rsakey.pem,$topfolder/keys/rsacert.pem,$topfolder/keys/ca2cert.pem" \
    "--CAfile $topfolder/keys/cacert.pem --keys-origins x509"

execDSigTest "aleksey-xmldsig-01/enveloping-hmac-ripemd160" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" 

execDSigTest "aleksey-xmldsig-01/enveloping-hmac-ripemd160-64" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" 

execDSigTest "aleksey-xmldsig-01/enveloping-hmac-md5" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" 

execDSigTest "aleksey-xmldsig-01/enveloping-hmac-md5-64" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" \
    "--hmackey keys/hmackey.bin" 
    
    
rm -rf $tmpfile
echo "--- testDSig finished" >> $logfile
echo "--- testDSig finished"
echo "--- detailed log is written to  $logfile" 

