#include <stdlib.h>
#include <stdio.h>

#include <libxml/debugXML.h>

#include <xmlsec/xmlsec.h>
#include <xmlsec/transforms.h>
#include <xmlsec/base64.h>
#include <xmlsec/c14n.h>
#include <xmlsec/sha1.h>

void test(void);

int main(int argc, char **argv) {

    fprintf(stderr, "Hello, world!\n");

    /*
     * Init libxml
     */     
    xmlInitParser();
    LIBXML_TEST_VERSION
    xmlSecInit();

    test();

    /* 
     * Shutdown libxml
     */
    xmlCleanupParser();
    
    return(0);
}

void test(void) {
    int res = 1;
    int ret;
    xmlSecTransformStatePtr state = NULL;
    xmlSecTransformPtr transform;
        
    state = xmlSecTransformStateCreate(NULL, NULL, "test.xml");
    if(state == NULL) {
	fprintf(stderr, "Error: failed to create state\n");
	goto done;
    }
    
    /*
    ret = xmlSecTransformStateFinal(state, xmlSecTransformResultXml);
    if(ret < 0) {
	fprintf(stderr, "Error: failed to create xml\n");
	goto done;	
    }
    
    if(state->curDoc == NULL) {
	fprintf(stderr, "Error: something is wrong: doc is null \n");
	goto done;	
    }    
    xmlDebugDumpDocument(stdout, state->curDoc);
    */
    
    /* encode */
    transform = xmlSecTransformCreate(xmlSecEncBase64Encode, 0, 0);
    if(transform == NULL) {
	fprintf(stderr, "Error: failed to create base64 transform\n");
	goto done;	
    }
    
    ret = xmlSecTransformStateUpdate(state, transform);
    if(ret < 0) {
	fprintf(stderr, "Error: failed to add base64 transform\n");
	goto done;	
    }    

    /* decode */
    transform = xmlSecTransformCreate(xmlSecEncBase64Decode, 0, 0);
    if(transform == NULL) {
	fprintf(stderr, "Error: failed to create base64 transform\n");
	goto done;	
    }
    
    ret = xmlSecTransformStateUpdate(state, transform);
    if(ret < 0) {
	fprintf(stderr, "Error: failed to add base64 transform\n");
	goto done;	
    }    
    
    /* c14n */
    transform = xmlSecTransformCreate(xmlSecC14NInclusive, 0, 0);
    if(transform == NULL) {
	fprintf(stderr, "Error: failed to create c14n transform\n");
	goto done;	
    }
    
    ret = xmlSecTransformStateUpdate(state, transform);
    if(ret < 0) {
	fprintf(stderr, "Error: failed to add c14n transform\n");
	goto done;	
    }    
        

    /* sha1 */
    transform = xmlSecTransformCreate(xmlSecDigestSha1, 0, 1);
    if(transform == NULL) {
	fprintf(stderr, "Error: failed to create sha1 transform\n");
	goto done;	
    }
    
    ret = xmlSecTransformStateUpdate(state, transform);
    if(ret < 0) {
	fprintf(stderr, "Error: failed to add sha1 transform\n");
	goto done;	
    }    

    ret = xmlSecTransformStateFinal(state, xmlSecTransformResultBinary);
    if(ret < 0) {
	fprintf(stderr, "Error: failed to create binary\n");
	goto done;	
    }
    
    
    {
	unsigned char *buf;
	size_t size;
	
	ret = xmlSecDigestSign(transform, &buf, &size);	
	if(ret < 0) {
	    fprintf(stderr, "Error: failed to sign\n");
	    xmlSecTransformDestroy(transform, 1);
	    goto done;	
	}
	
	fwrite(buf, size, 1, stdout);	
	xmlSecTransformDestroy(transform, 1);
    }
/*
    if(state->curBuf == NULL) {
	fprintf(stderr, "Error: something is wrong: buf is null \n");
	goto done;	
    }    
    fwrite(xmlBufferContent(state->curBuf), xmlBufferLength(state->curBuf), 1, stdout);
*/
    res = 0;
done:
    if(state != NULL) {
	xmlSecTransformStateDestroy(state);
    }
}

